@@ -202,7 +202,7 @@ typedef struct pinentry *pinentry_t;
    entry.  On confirmation, the function should return TRUE if
    confirmed, and FALSE otherwise.  On PIN entry, the function should
    return -1 if an error occured or the user cancelled the operation
-   and the length of the secret otherwise.  */
+   and 1 otherwise.  */
 typedef int (*pinentry_cmd_handler_t) (pinentry_t pin);
 
 /* Start the pinentry event loop.  The program will start to process
@@ -211,7 +211,10 @@ typedef int (*pinentry_cmd_handler_t) (pinentry_t pin);
    error.  Otherwise, 0 is returned.  */
 int pinentry_loop (void);
 
-/* The same as above but allows to specify the i/o descriptors. */
+/* The same as above but allows to specify the i/o descriptors.
+ * infd and outfd will be duplicated in this function so the caller
+ * still has to close them if necessary.
+ */
 int pinentry_loop2 (int infd, int outfd);
 
 
