@@ -1,9 +1,9 @@
 /* util.h - Utility functions for GnuPG
  * Copyright (C) 2001, 2002, 2003, 2004, 2009 Free Software Foundation, Inc.
  *
- * This file is part of JNLIB, which is a subsystem of GnuPG.
+ * This file is part of GnuPG.
  *
- * JNLIB is free software; you can redistribute it and/or modify it
+ * GnuPG is free software; you can redistribute it and/or modify it
  * under the terms of either
  *
  *   - the GNU Lesser General Public License as published by the Free
@@ -18,7 +18,7 @@
  *
  * or both in parallel, as here.
  *
- * JNLIB is distributed in the hope that it will be useful, but
+ * GnuPG is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -35,6 +35,100 @@
 #include <errno.h>  /* We need errno.  */
 #include <gpg-error.h> /* We need gpg_error_t and estream. */
 
+/* These error codes are used but not defined in the required
+   libgpg-error version.  Define them here. */
+#if GPG_ERROR_VERSION_NUMBER < 0x011200  /* 1.18 */
+# define GPG_ERR_LEGACY_KEY     222
+# define GPG_ERR_OBJ_TERM_STATE 225
+# define GPG_ERR_FORBIDDEN      251
+#endif
+#if GPG_ERROR_VERSION_NUMBER < 0x011300  /* 1.19 */
+# define GPG_ERR_LDAP_GENERAL               721
+# define GPG_ERR_LDAP_ATTR_GENERAL          722
+# define GPG_ERR_LDAP_NAME_GENERAL          723
+# define GPG_ERR_LDAP_SECURITY_GENERAL      724
+# define GPG_ERR_LDAP_SERVICE_GENERAL       725
+# define GPG_ERR_LDAP_UPDATE_GENERAL        726
+# define GPG_ERR_LDAP_E_GENERAL             727
+# define GPG_ERR_LDAP_X_GENERAL             728
+# define GPG_ERR_LDAP_OTHER_GENERAL         729
+# define GPG_ERR_LDAP_X_CONNECTING          750
+# define GPG_ERR_LDAP_REFERRAL_LIMIT        751
+# define GPG_ERR_LDAP_CLIENT_LOOP           752
+# define GPG_ERR_LDAP_NO_RESULTS            754
+# define GPG_ERR_LDAP_CONTROL_NOT_FOUND     755
+# define GPG_ERR_LDAP_NOT_SUPPORTED         756
+# define GPG_ERR_LDAP_CONNECT               757
+# define GPG_ERR_LDAP_NO_MEMORY             758
+# define GPG_ERR_LDAP_PARAM                 759
+# define GPG_ERR_LDAP_USER_CANCELLED        760
+# define GPG_ERR_LDAP_FILTER                761
+# define GPG_ERR_LDAP_AUTH_UNKNOWN          762
+# define GPG_ERR_LDAP_TIMEOUT               763
+# define GPG_ERR_LDAP_DECODING              764
+# define GPG_ERR_LDAP_ENCODING              765
+# define GPG_ERR_LDAP_LOCAL                 766
+# define GPG_ERR_LDAP_SERVER_DOWN           767
+# define GPG_ERR_LDAP_SUCCESS               768
+# define GPG_ERR_LDAP_OPERATIONS            769
+# define GPG_ERR_LDAP_PROTOCOL              770
+# define GPG_ERR_LDAP_TIMELIMIT             771
+# define GPG_ERR_LDAP_SIZELIMIT             772
+# define GPG_ERR_LDAP_COMPARE_FALSE         773
+# define GPG_ERR_LDAP_COMPARE_TRUE          774
+# define GPG_ERR_LDAP_UNSUPPORTED_AUTH      775
+# define GPG_ERR_LDAP_STRONG_AUTH_RQRD      776
+# define GPG_ERR_LDAP_PARTIAL_RESULTS       777
+# define GPG_ERR_LDAP_REFERRAL              778
+# define GPG_ERR_LDAP_ADMINLIMIT            779
+# define GPG_ERR_LDAP_UNAVAIL_CRIT_EXTN     780
+# define GPG_ERR_LDAP_CONFIDENT_RQRD        781
+# define GPG_ERR_LDAP_SASL_BIND_INPROG      782
+# define GPG_ERR_LDAP_NO_SUCH_ATTRIBUTE     784
+# define GPG_ERR_LDAP_UNDEFINED_TYPE        785
+# define GPG_ERR_LDAP_BAD_MATCHING          786
+# define GPG_ERR_LDAP_CONST_VIOLATION       787
+# define GPG_ERR_LDAP_TYPE_VALUE_EXISTS     788
+# define GPG_ERR_LDAP_INV_SYNTAX            789
+# define GPG_ERR_LDAP_NO_SUCH_OBJ           800
+# define GPG_ERR_LDAP_ALIAS_PROBLEM         801
+# define GPG_ERR_LDAP_INV_DN_SYNTAX         802
+# define GPG_ERR_LDAP_IS_LEAF               803
+# define GPG_ERR_LDAP_ALIAS_DEREF           804
+# define GPG_ERR_LDAP_X_PROXY_AUTH_FAIL     815
+# define GPG_ERR_LDAP_BAD_AUTH              816
+# define GPG_ERR_LDAP_INV_CREDENTIALS       817
+# define GPG_ERR_LDAP_INSUFFICIENT_ACC      818
+# define GPG_ERR_LDAP_BUSY                  819
+# define GPG_ERR_LDAP_UNAVAILABLE           820
+# define GPG_ERR_LDAP_UNWILL_TO_PERFORM     821
+# define GPG_ERR_LDAP_LOOP_DETECT           822
+# define GPG_ERR_LDAP_NAMING_VIOLATION      832
+# define GPG_ERR_LDAP_OBJ_CLS_VIOLATION     833
+# define GPG_ERR_LDAP_NOT_ALLOW_NONLEAF     834
+# define GPG_ERR_LDAP_NOT_ALLOW_ON_RDN      835
+# define GPG_ERR_LDAP_ALREADY_EXISTS        836
+# define GPG_ERR_LDAP_NO_OBJ_CLASS_MODS     837
+# define GPG_ERR_LDAP_RESULTS_TOO_LARGE     838
+# define GPG_ERR_LDAP_AFFECTS_MULT_DSAS     839
+# define GPG_ERR_LDAP_VLV                   844
+# define GPG_ERR_LDAP_OTHER                 848
+# define GPG_ERR_LDAP_CUP_RESOURCE_LIMIT    881
+# define GPG_ERR_LDAP_CUP_SEC_VIOLATION     882
+# define GPG_ERR_LDAP_CUP_INV_DATA          883
+# define GPG_ERR_LDAP_CUP_UNSUP_SCHEME      884
+# define GPG_ERR_LDAP_CUP_RELOAD            885
+# define GPG_ERR_LDAP_CANCELLED             886
+# define GPG_ERR_LDAP_NO_SUCH_OPERATION     887
+# define GPG_ERR_LDAP_TOO_LATE              888
+# define GPG_ERR_LDAP_CANNOT_CANCEL         889
+# define GPG_ERR_LDAP_ASSERTION_FAILED      890
+# define GPG_ERR_LDAP_PROX_AUTH_DENIED      891
+#endif /*GPG_ERROR_VERSION_NUMBER < 0x011300*/
+#if GPG_ERROR_VERSION_NUMBER < 0x011500  /* 1.21 */
+# define GPG_ERR_TRUE                       255
+# define GPG_ERR_FALSE                      256
+#endif
 
 /* Hash function used with libksba. */
 #define HASH_FNC ((void (*)(void *, const void*,size_t))gcry_md_write)
@@ -62,12 +156,30 @@
 #define snprintf gpgrt_snprintf
 
 
-/* GCC attributes.  */
-#if __GNUC__ >= 4
-# define GNUPG_GCC_A_SENTINEL(a) __attribute__ ((sentinel(a)))
-#else
-# define GNUPG_GCC_A_SENTINEL(a)
-#endif
+/* Replacements for macros not available with libgpg-error < 1.20.  */
+#ifndef GPGRT_GCC_VERSION
+
+# ifndef GPGRT_ATTR_FORMAT_ARG
+# if __GNUC__ >= 3 /* Actually 2.8 but testing the major is easier.  */
+#  define GPGRT_ATTR_FORMAT_ARG(a)  __attribute__ ((__format_arg__ (a)))
+# else
+#  define GPGRT_ATTR_FORMAT_ARG(a)
+# endif
+# endif
+
+# if __GNUC__ >= 4
+#  define GPGRT_ATTR_SENTINEL(a) __attribute__ ((sentinel(a)))
+# else
+#  define GPGRT_ATTR_SENTINEL(a)
+# endif
+
+# if __GNUC__ >= 4
+#  define GPGRT_ATTR_USED __attribute__ ((used))
+# else
+#  define GPGRT_ATTR_USED
+# endif
+
+#endif /*libgpg-error < 1.20 */
 
 
 /* We need this type even if we are not using libreadline and or we
@@ -214,8 +326,10 @@ size_t percent_unescape_inplace (char *string, int nulrepl);
 gpg_error_t openpgp_oid_from_str (const char *string, gcry_mpi_t *r_mpi);
 char *openpgp_oid_to_str (gcry_mpi_t a);
 int openpgp_oid_is_ed25519 (gcry_mpi_t a);
+int openpgp_oid_is_crv25519 (gcry_mpi_t a);
 const char *openpgp_curve_to_oid (const char *name, unsigned int *r_nbits);
-const char *openpgp_oid_to_curve (const char *oid);
+const char *openpgp_oid_to_curve (const char *oid, int canon);
+const char *openpgp_enum_curves (int *idxp);
 
 
 
@@ -246,6 +360,7 @@ const char *dirmngr_user_socket_name (void);
 #define GNUPG_MODULE_NAME_GPGCONF       10
 #define GNUPG_MODULE_NAME_DIRMNGR_LDAP  11
 const char *gnupg_module_name (int which);
+void gnupg_module_name_flush_some (void);
 
 
 
@@ -265,13 +380,16 @@ const char *gnupg_messages_locale_name (void);
 void setup_libgcrypt_logging (void);
 
 /* Same as estream_asprintf but die on memory failure.  */
-char *xasprintf (const char *fmt, ...) JNLIB_GCC_A_PRINTF(1,2);
+char *xasprintf (const char *fmt, ...) GPGRT_ATTR_PRINTF(1,2);
 /* This is now an alias to estream_asprintf.  */
-char *xtryasprintf (const char *fmt, ...) JNLIB_GCC_A_PRINTF(1,2);
+char *xtryasprintf (const char *fmt, ...) GPGRT_ATTR_PRINTF(1,2);
 
 /* Replacement for gcry_cipher_algo_name.  */
 const char *gnupg_cipher_algo_name (int algo);
 
+void obsolete_option (const char *configname, unsigned int configlineno,
+                      const char *name);
+
 const char *print_fname_stdout (const char *s);
 const char *print_fname_stdin (const char *s);
 void print_utf8_buffer3 (estream_t fp, const void *p, size_t n,
@@ -288,6 +406,14 @@ int match_multistr (const char *multistr,const char *match);
 
 int gnupg_compare_version (const char *a, const char *b);
 
+struct debug_flags_s
+{
+  unsigned int flag;
+  const char *name;
+};
+int parse_debug_flag (const char *string, unsigned int *debugvar,
+                      const struct debug_flags_s *flags);
+
 
 /*-- Simple replacement functions. */
 
